import { r as registerInstance, c as createEvent, h, g as getElement } from './p-597600b0.js';
import { g as getControlSection, l as loadAndUpdatePageConfig, u as updateCurrentTabWithMappingsFromOtherTabs, c as updateResultants, d as getOutputSubCategoriesObj, e as updateSelectedOutputSubCategoryId, f as getOutputSection, h as getTabNamesObj, i as updateSelectedTabId, n as getIDValue, s as showPopup, j as getElementKey, k as updateConfigData } from './p-31525cd2.js';
import { k as CRC_IMPLEMENTATION_DIAGRAM_ID, c as configImageDirPath } from './p-3ab399ac.js';
import { g as getTableRow, d as decToBinArrMSBToLSB, a as decToHexStr } from './p-d57b9fa3.js';

const adsCrcCss = ":root{font-family:Roboto}.d-none{display:none}.v-hidden{visibility:hidden}.c-pointer{cursor:pointer}.disabled{cursor:not-allowed;opacity:0.6}.fw-bold{font-weight:bold}::-webkit-scrollbar-track{background-color:var(--color-ti-bg-grey);border-radius:5px}::-webkit-scrollbar{background-color:var(--color-ti-bg-grey);height:5px;width:5px}::-webkit-scrollbar-thumb{background-color:var(--color-ti-text-grey);border-radius:5px}ads-popup::part(popup-container){height:75%;width:75%}.page-header{box-sizing:content-box;display:flex;height:28px;justify-content:space-between;padding-bottom:5px}.page-header .page-title{color:var(--color-ti-text-grey);font-size:24px}.page-content{display:flex;height:calc(100% - 33px)}.page-content .left-section,.page-content .right-section{display:flex;flex-direction:column;gap:2%;height:100%;min-height:670px;padding:2%}.page-content .left-section{background-color:var(--color-ti-primary-white);min-width:540px;width:40%}.page-content .right-section{background-color:#f2f8fa;min-width:720px;width:60%}.popup-btn{align-items:center;cursor:pointer;display:flex;text-decoration:underline}.popup-content{display:flex;flex-direction:column}.popup-content img{margin:auto;max-height:calc(100% - 100px);max-width:calc(100% - 100px)}.popup-content .title{background-color:var(--color-ti-indicator-grey);font-size:1.5rem;font-weight:bold;height:40px;margin:0 auto;padding:5px;text-align:center;width:50%}.section-title{color:var(--color-ti-text-blue);font-size:18px}.section-tabs,.section-outputs{display:flex;flex-direction:column;height:100%;overflow:auto;width:100%}.section-outputs ads-tabs{--selected-tab-color:var(--color-ti-primary-blue-dark)}.section-outputs .tab-content{display:flex;flex-direction:column}.tab-content{height:max-content;overflow:hidden auto;width:100%}.tab-content ads-control{--control-label-width:50%}.tab-content .group-title{color:var(--color-ti-primary-red);font-size:1.2em;margin-top:5%}*{box-sizing:border-box}:host{background-color:var(--color-ti-bg-grey-dark);display:flex;flex-direction:column;height:100%;overflow:auto;padding:2%;width:100%}:host ads-popup::part(popup-container){height:60%}.error-text{display:flex;height:25%;width:100%;align-items:center;justify-content:center;font-weight:bold;font-size:25px;opacity:0.3}.error-icon{--gc-color:var(--color-ti-primary-red)}";

const AdsCrc = class {
  constructor(hostRef) {
    registerInstance(this, hostRef);
    this.showToast = createEvent(this, "showToast", 7);
    // ---------- Internal variables ----------
    this.computationAPIs = {};
    this.isComputedValuesInitialized = false;
    // ----------- Helper methods -------------
    /**
     * To get control element
     * @param controlInfo
     * @param dataPath
     * @returns Control element
     */
    this.getControlElement = (controlInfo, dataPath) => {
      return (h("ads-control", { key: getElementKey(controlInfo), controlInfo: controlInfo, dataPath: dataPath, onSetPathValue: ev => {
          let canUpdateResultant = ev.detail.dataPath[4] === "multiLine_hex_data" ? false : true;
          updateConfigData.call(this, ev, canUpdateResultant);
          if (ev.detail.dataPath[4] === "crc_polynomial") {
            setTimeout(() => {
              this.showToast.emit({
                message: `CRC for ${this.config.tabs.configuration.controls.crc_polynomial
                  .options[ev.detail.value]} polynomial is computed`,
              });
            }, 1000);
          }
        } }));
    };
    /**
     * To get control groups
     * @returns Control groups
     */
    this.getControlSectionGroups = () => {
      const groupsInfo = getControlSection.call(this);
      let groups = [];
      groupsInfo.forEach(group => {
        groups.push(h("div", null, group.title ? h("div", { class: "group-title" }, group.title) : null, ...group.controls));
      });
      return groups;
    };
    /**
     * To get value for CRC table
     * @param metaData
     * @returns CRC table value object
     */
    this.getValueForCRCTable = metaData => {
      let { crcInitValue, polynomial, crcLength, statusBitsArr, dataBitsArr, crcChangesArr, } = metaData;
      const CSS_CLASS_NAMES = {
        binaryValue: "binary-value",
        boldText: "fw-bold",
        crcBitIndex: "crc-bit-index",
        crcResult: "crc-result",
        cycle: "cycle",
        finalValue: "final-value",
        unsetTransform: "unset-transform",
        verticalText: "vertical-text",
        whiteCell: "white-cell",
        minHeight: "min-height",
        xorCell: "xor-cell",
      };
      let crcTable = [
        // Header row - 1st row
        getTableRow([
          { rowspan: 2 },
          {
            value: "Binary Value",
            classList: [
              CSS_CLASS_NAMES.binaryValue,
              CSS_CLASS_NAMES.verticalText,
            ],
            rowspan: 2,
          },
          {
            value: "Cycle",
            classList: [CSS_CLASS_NAMES.cycle, CSS_CLASS_NAMES.verticalText],
            rowspan: 2,
          },
          // CRC MSB
          {
            classList: [
              CSS_CLASS_NAMES.crcBitIndex,
              CSS_CLASS_NAMES.verticalText,
            ],
            value: "MSB",
          },
          // CRC bit indexes (except for MSB & LSB)
          ...decToBinArrMSBToLSB(crcInitValue, crcLength)
            .slice(1, -1)
            .map((bit, index) => {
            return {
              value: crcLength - 2 - index,
              classList: [CSS_CLASS_NAMES.crcBitIndex],
            };
          }),
          // CRC LSB
          {
            value: "LSB",
            classList: [
              CSS_CLASS_NAMES.crcBitIndex,
              CSS_CLASS_NAMES.verticalText,
            ],
          },
          {
            classList: [CSS_CLASS_NAMES.crcBitIndex],
          },
        ], "head"),
        // Header row - 2nd row
        getTableRow([
          // CRC Bit Values (Initial Value)
          ...decToBinArrMSBToLSB(crcInitValue, crcLength).map(bit => {
            return {
              value: bit,
            };
          }),
          {
            value: "← Initial Seed",
          },
        ], "head"),
      ];
      crcChangesArr.forEach((crcValue, dataBitIndex) => {
        let firstColumnCell = [];
        let lastColumnCell = [];
        switch (true) {
          case dataBitIndex === 0:
            if (statusBitsArr === undefined) {
              firstColumnCell.push({
                value: "Data MSB",
                rowspan: 1,
                classList: [CSS_CLASS_NAMES.boldText],
              });
            }
            else {
              firstColumnCell.push({
                value: "Status MSB",
                rowspan: 1,
                classList: [CSS_CLASS_NAMES.boldText],
              });
            }
            lastColumnCell.push({
              value: "",
              rowspan: crcChangesArr.length - 1,
              classList: [CSS_CLASS_NAMES.whiteCell],
            });
            break;
          case dataBitIndex === 1:
            if (statusBitsArr === undefined) {
              firstColumnCell.push({
                value: "",
                rowspan: dataBitsArr.length - 2,
              });
            }
            else {
              firstColumnCell.push({
                value: "",
                rowspan: statusBitsArr.length - 1,
              });
            }
            break;
          case statusBitsArr !== undefined &&
            dataBitIndex === statusBitsArr.length:
            firstColumnCell.push({
              value: "Data MSB",
              rowspan: 1,
              classList: [CSS_CLASS_NAMES.boldText],
            });
            break;
          case statusBitsArr !== undefined &&
            dataBitIndex === statusBitsArr.length + 1:
            firstColumnCell.push({
              value: "",
              rowspan: dataBitsArr.length - 2,
            });
            break;
          case dataBitIndex === crcChangesArr.length - 1:
            firstColumnCell.push({
              value: "CRC Result",
            });
            lastColumnCell.push({
              value: decToHexStr(crcValue, crcLength),
              classList: [CSS_CLASS_NAMES.crcResult, CSS_CLASS_NAMES.finalValue],
            });
            break;
        }
        crcTable.push(getTableRow([
          ...firstColumnCell,
          // Binary value
          {
            classList: [CSS_CLASS_NAMES.binaryValue],
            value: statusBitsArr === undefined
              ? dataBitsArr[dataBitIndex]
              : dataBitIndex >= statusBitsArr.length
                ? dataBitsArr[dataBitIndex - statusBitsArr.length]
                : statusBitsArr[dataBitIndex],
          },
          // Cycle
          {
            classList: [CSS_CLASS_NAMES.cycle],
            value: dataBitIndex + 1,
          },
          // CRC Bit Value (Current Cycle)
          ...decToBinArrMSBToLSB(crcValue, crcLength).map((crcBitValue, crcBitIndex) => {
            return {
              classList: [
                dataBitIndex === crcChangesArr.length - 1
                  ? CSS_CLASS_NAMES.crcResult
                  : ((polynomial >> (crcLength - 1 - crcBitIndex)) & 1) > 0
                    ? CSS_CLASS_NAMES.xorCell
                    : "",
              ],
              value: crcBitValue,
            };
          }),
          // CRC Result
          ...lastColumnCell,
        ], dataBitIndex === crcChangesArr.length - 1 ? "foot" : undefined));
      });
      return crcTable;
    };
    /**
     * To get output info
     * @param outputInfo Output info object
     * @returns Output element
     */
    this.getOutputElement = (outputInfo, dataPath) => {
      if (outputInfo.outputType === "table" &&
        outputInfo.metaData !== undefined) {
        if (outputInfo.metaData.isTableRequired) {
          outputInfo = Object.assign(Object.assign({}, outputInfo), { value: this.getValueForCRCTable(outputInfo.metaData) });
        }
        else {
          return (h("div", { class: "error-text" }, "Table is NA for hex data more than 50 bytes", " ", h("gc-widget-icon", { id: "tooltip", appearance: "custom", icon: "outlined:alert:warning", size: "m", class: "error-icon" })));
        }
      }
      return (h("ads-output", { "selected-device": this.selectedDevice, "tool-name": "crc", key: getElementKey(outputInfo), outputInfo: outputInfo, dataPath: dataPath, onSetPathValue: ev => updateConfigData.call(this, ev), "reset-min-height": outputInfo.outputType === "table" }));
    };
    this.selectedDevice = undefined;
    this.selectedPageIndex = undefined;
    this.selectedToolName = undefined;
    this.config = undefined;
    this.selectedTabId = undefined;
    this.selectedOutputSubCategoryId = undefined;
    this.isShowPopup = false;
  }
  // ---------- Life cycle methods ----------
  componentDidLoad() {
    if (this.config === undefined && this.selectedDevice !== undefined) {
      this.loadPageConfig();
    }
  }
  // ----------- Watch handlers -------------
  loadPageConfig() {
    loadAndUpdatePageConfig.call(this, this.selectedToolName, this.selectedDevice);
  }
  onSelectedTabChanged() {
    if (this.config != undefined) {
      updateCurrentTabWithMappingsFromOtherTabs.call(this);
      updateResultants.call(this, true);
    }
  }
  render() {
    return this.config != undefined && this.isComputedValuesInitialized
      ? [
        h("div", { class: "page-header" }, h("div", { class: "page-title" }, "CRC Tool"), h("ads-dropdown-menu", { name: "Related links", items: this.config.collaterals })),
        h("div", { class: "page-content" }, h("div", { class: "left-section", id: "left-section" }, h("div", { class: "section-title" }, this.config.label), h("div", { class: "section-tabs" }, h("ads-tabs", { tabNamesObj: getTabNamesObj(this.config.tabs), selectedTabId: this.selectedTabId, onUpdateSelectedTab: ev => updateSelectedTabId.call(this, ev), selectedPageIndex: this.selectedPageIndex, "hide-if-single-tab": true }), h("div", { class: "tab-content" }, this.getControlSectionGroups())), h("div", null, getIDValue.call(this, CRC_IMPLEMENTATION_DIAGRAM_ID) === "" ||
          getIDValue.call(this, CRC_IMPLEMENTATION_DIAGRAM_ID) ===
            undefined ? null : (h("div", { class: "popup-btn", onClick: ev => showPopup.call(this, ev) }, h("gc-widget-icon", { icon: "filled:action:info_outline", size: "s" }), "Click here to see the CRC implementation")))), h("div", { class: "right-section", id: "right-section" }, h("div", { class: "section-title" }, "Resultant Calculations"), h("div", { class: "section-outputs" }, this.selectedTabId != undefined && (h("ads-tabs", { tabNamesObj: getOutputSubCategoriesObj.call(this), selectedTabId: this.selectedOutputSubCategoryId, selectedPageIndex: this.selectedPageIndex, onUpdateSelectedTab: ev => updateSelectedOutputSubCategoryId.call(this, ev), "hide-if-single-tab": true })), h("div", { class: "tab-content" }, getOutputSection.call(this))))),
        h("div", null, getIDValue.call(this, CRC_IMPLEMENTATION_DIAGRAM_ID) === "" ||
          getIDValue.call(this, CRC_IMPLEMENTATION_DIAGRAM_ID) ===
            undefined ? null : (h("ads-popup", { isShowPopup: this.isShowPopup, onPopupClosed: () => (this.isShowPopup = false), ref: el => (this.popupDiv = el) }, h("div", { class: "popup-content", ref: el => (this.popupContentDiv = el), slot: "content" }, h("div", { class: "title" }, "CRC Implementation"), h("img", { alt: "Implementation diagram", src: `${configImageDirPath}${getIDValue.call(this, CRC_IMPLEMENTATION_DIAGRAM_ID)}` }))))),
      ]
      : [];
  }
  get host() { return getElement(this); }
  static get watchers() { return {
    "selectedDevice": ["loadPageConfig"],
    "selectedTabId": ["onSelectedTabChanged"]
  }; }
};
AdsCrc.style = adsCrcCss;

export { AdsCrc as ads_crc };

//# sourceMappingURL=p-24ba1583.entry.js.map