import * as utility from "./toolUtil.js";

// ---------- Controls & constants ----------
export function updateCrcPolynomial(args) {
  let configData = args.parameterConfigData[args.selectedDevice];
  let polynomialValue = get_Polynomial_Value(
    configData,
    args.crc_polynomial.value,
  );
  return {
    options: get_Polynomial_List(configData),
    value: polynomialValue,
    size: get_Polynomial_Size(configData, polynomialValue),
    initialValue: get_Initial_Value(configData, polynomialValue),
  };
}

export function updateCrcPolynomialValue(args) {
  return {
    value: utility.decToHexStr(
      args.crc_polynomial.value,
      args.crc_polynomial.size,
    ),
  };
}

export function updateHexData(args) {
  return {
    value: args.multiLine_hex_data.value,
    size: args.multiLine_hex_data.value.length - 2,
  };
}

export function updateImplementationDiagramFileName(args) {
  let configData = args.parameterConfigData[args.selectedDevice];
  return {
    value: get_Implementation_Diagram_FileName(
      configData,
      args.crc_polynomial.value,
    ),
  };
}

// ---------- Outputs ----------

export function computeCrcResult(args) {
  let resultDetails = computeCrcTable(args);

  return {
    value: utility.decToHexStr(
      resultDetails.value,
      resultDetails.metaData.crcLength,
    ),
  };
}

export function computeCrcTable(args) {
  // Parse
  const crcInitValue = parseInt(args.crc_polynomial.initialValue, 16);
  const polynomial = args.crc_polynomial.value;
  const hexData = args.multiLine_hex_data.value.replaceAll("0x", "");
  const crcLength = parseInt(args.crc_polynomial.size);

  const statusBitsArr = undefined;
  let binaryString = "";
  for (let i = 0; i < hexData.length; i++) {
    const hexChar = hexData.charAt(i);
    const binaryValue = parseInt(hexChar, 16).toString(2);
    const paddedBinaryValue =
      "0000".substring(binaryValue.length) + binaryValue;
    binaryString += paddedBinaryValue;
  }
  const dataBitsArr = binaryString.split("").map(Number);

  let crcValue = crcInitValue;
  let crcChangesArr = [];

  // Compute
  for (
    let dataBitIndex = 0;
    dataBitIndex < dataBitsArr.length;
    dataBitIndex++
  ) {
    const dataMSb = Boolean(dataBitsArr[dataBitIndex]);
    const crcMSb = get_Crc_Msb(crcValue, crcLength);

    crcValue <<= 1;

    if (Boolean(dataMSb ^ crcMSb)) {
      crcValue ^= polynomial;
    }

    //Filling the missing bits with 0
    crcValue = get_Valid_Crc_Value(crcValue, crcLength);

    crcChangesArr.push(crcValue);
  }

  return {
    value: crcValue,
    metaData: {
      crcInitValue,
      polynomial,
      crcLength,
      statusBitsArr,
      dataBitsArr,
      crcChangesArr,
      isTableRequired: hexData.length <= 100,
    },
  };
}

function get_Polynomial_List(configData) {
  return Object.assign(
    {},
    ...configData.map(data => ({
      [parseInt(data.polynomial, 16)]: data.displayName,
    })),
  );
}

function get_Polynomial_Value(configData, polynomialValue) {
  if (polynomialValue) {
    if (
      typeof polynomialValue === "string" &&
      (polynomialValue.includes("0x") ||
        polynomialValue.includes("h", polynomialValue.length - 1))
    ) {
      return parseInt(polynomialValue, 16);
    }
    return polynomialValue;
  }
  return parseInt(configData[0].polynomial, 16);
}

function get_Polynomial_Size(configData, polynomialValue) {
  if (polynomialValue) {
    return configData.filter(
      data => parseInt(data.polynomial, 16) === polynomialValue,
    )[0].crcBitWidth;
  }
  return configData[0].crcBitWidth;
}

function get_Initial_Value(configData, polynomialValue) {
  if (polynomialValue) {
    return configData.filter(
      data => parseInt(data.polynomial, 16) === polynomialValue,
    )[0].seedValue;
  }
  return configData[0].seedValue;
}

function get_Implementation_Diagram_FileName(configData, polynomialValue) {
  if (polynomialValue) {
    return configData.filter(
      data => parseInt(data.polynomial, 16) === polynomialValue,
    )[0].implementationDiagramFileName;
  }
  return configData[0].implementationDiagramFileName;
}

function get_Crc_Msb(crcValue, crcBitWidth) {
  switch (crcBitWidth) {
    case 4:
      return Boolean(crcValue & 0x8);
    case 8:
      return Boolean(crcValue & 0x80);
    case 16:
      return Boolean(crcValue & 0x8000);
    default:
      return undefined;
  }
}

function get_Valid_Crc_Value(crcValue, crcBitWidth) {
  switch (crcBitWidth) {
    case 4:
      return crcValue & 0xf;
    case 8:
      return crcValue & 0xff;
    case 16:
      return crcValue & 0xffff;
    default:
      return undefined;
  }
}
