import * as utility from "./toolUtil.js";

// ---------- Controls & constants ----------
export function updateCrcPolynomial(args) {
  let configData = args.parameterConfigData[args.selectedDevice];
  let polynomialValue = get_Polynomial_Value(
    configData,
    args.crc_polynomial.value,
  );
  return {
    options: get_Polynomial_List(configData),
    value: polynomialValue,
    size: get_Polynomial_Size(configData, polynomialValue),
    initialValue: get_Initial_Value(configData, polynomialValue),
  };
}

export function updateCrcPolynomialValue(args) {
  return {
    value: utility.decToHexStr(
      args.crc_polynomial.value,
      args.crc_polynomial.size,
    ),
  };
}

export function updateStatusDataInput(args) {
  return {
    isDisabled: !args.status_data_status.value,
    value: args.status_data_status.value ? args.status_data.value : "0x00",
  };
}

export function updateImplementationDiagramFileName(args) {
  let configData = args.parameterConfigData[args.selectedDevice];
  return {
    value: get_Implementation_Diagram_FileName(
      configData,
      args.crc_polynomial.value,
    ),
  };
}

// ---------- Outputs ----------

export function computeCrcResult(args) {
  let resultDetails = computeCrcTable(args);

  return {
    value: utility.decToHexStr(
      resultDetails.value,
      resultDetails.metaData.crcLength,
    ),
  };
}

export function computeCrcTable(args) {
  // Parse
  const isStatusByteEnabled = args.status_data_status.value;
  const crcInitValue = parseInt(args.crc_polynomial.initialValue, 16);
  const polynomial = args.crc_polynomial.value;
  const hexData = parseInt(args.hex_data.value);
  const statusData = parseInt(args.status_data.value);
  const crcLength = parseInt(args.crc_polynomial.size);


  // Initialize
  const statusBitsArr = isStatusByteEnabled
    ? utility.decToBinArrMSBToLSB(statusData, args.status_data.size * 4)
    : undefined;
  const dataBitsArr = utility.decToBinArrMSBToLSB(
    hexData,
    args.hex_data.size * 4,
  );
  const statusWithData = isStatusByteEnabled
    ? [...statusBitsArr, ...dataBitsArr]
    : dataBitsArr;

  let crcValue = crcInitValue;
  let crcChangesArr = [];

  // Compute
  for (
    let dataBitIndex = 0;
    dataBitIndex < statusWithData.length;
    dataBitIndex++
  ) {
    const dataMSb = Boolean(statusWithData[dataBitIndex]);
    const crcMSb = Boolean(crcValue & 0x80);

    crcValue <<= 1;

    if (Boolean(dataMSb ^ crcMSb)) {
      crcValue ^= polynomial;
    }

    crcValue &= 0xff;

    crcChangesArr.push(crcValue);
  }

  return {
    value: crcValue,
    metaData: {
      crcInitValue,
      polynomial,
      crcLength,
      statusBitsArr,
      dataBitsArr,
      crcChangesArr,
      isTableRequired: true,
    },
  };
}

function get_Polynomial_List(configData) {
  return Object.assign(
    {},
    ...configData.map(data => ({
      [parseInt(data.polynomial, 16)]: data.displayName,
    })),
  );
}

function get_Polynomial_Value(configData, polynomialValue) {
  if (polynomialValue) {
    if (
      typeof polynomialValue === "string" &&
      (polynomialValue.includes("0x") ||
        polynomialValue.includes("h", polynomialValue.length - 1))
    ) {
      return parseInt(polynomialValue, 16);
    }
    return polynomialValue;
  }
  return parseInt(configData[0].polynomial, 16);
}

function get_Polynomial_Size(configData, polynomialValue) {
  if (polynomialValue) {
    return configData.filter(
      data => parseInt(data.polynomial, 16) === polynomialValue,
    )[0].crcBitWidth;
  }
  return configData[0].crcBitWidth;
}

function get_Initial_Value(configData, polynomialValue) {
  if (polynomialValue) {
    return configData.filter(
      data => parseInt(data.polynomial, 16) === polynomialValue,
    )[0].seedValue;
  }
  return configData[0].seedValue;
}

function get_Implementation_Diagram_FileName(configData, polynomialValue) {
  if (polynomialValue) {
    return configData.filter(
      data => parseInt(data.polynomial, 16) === polynomialValue,
    )[0].implementationDiagramFileName;
  }
  return configData[0].implementationDiagramFileName;
}
