import { Config } from "@stencil/core";
import { sass } from "@stencil/sass";

export const config: Config = {
  namespace: "padc_design_calculator_tool",
  globalStyle: "./src/styles/vars.scss",
  srcDir: "./src",
  sourceMap: true,
  minifyJs: false,
  minifyCss: false,
  plugins: [
    sass({}),
    {
      name: "externalLibs",
      resolveId(id: string, importer?: string) {
        // Paths to ignore in rollup
        const EXTERNAL_LIB_REGEX = /.\/libs\/.*.js/;
        const GC_REGEX = /.*components[/]@ti[/][^/]+[/]lib[/][^/]+/;
        const isGcComponent = GC_REGEX.test(id);
        const noRollup = EXTERNAL_LIB_REGEX.test(id) || isGcComponent;

        // if (noRollup) {
        //     console.log('Rollup is ignoring ', id);
        // }

        if (isGcComponent) {
          if (id.includes("../../../../../../designer/v3/components/@ti")) {
            id = id.replace(
              "../../../../../../designer/v3/components/@ti",
              "../../components/@ti",
            );
          } else if (id.includes("../../../../../designer/v3/components/@ti")) {
            id = id.replace(
              "../../../../../designer/v3/components/@ti",
              "../../components/@ti",
            );
          } else {
            id = id.replace(
              "../../../../designer/v3/components/@ti",
              "../../components/@ti",
            );
          }
        }

        return noRollup ? { id: id, external: true } : null;
      },
    },
  ],
  outputTargets: [
    {
      type: "www",
      dir: "dist",
      copy: [
        {
          src: "config{/!(digitalFilterCalculationFiles)/**,/*}",
        },
        {
          src: "config{/!(codeConversionCalculationFiles)/**,/*}",
        },
        {
          src: "components/pages/**/*.{json,js}",
          dest: "files",
        },
        {
          src: "components/**/*.svg",
          dest: "assets/images",
        },
        {
          src: "components/pages/**/*.{png,jpg}",
          dest: "config/assets",
        },
        {
          src: "utilities/toolUtil.{js,d.ts}",
          dest: "files",
        },
        {
          src: "config/digitalFilterCalculationFiles",
          dest: "digitalFilterCalculationFiles",
        },
        {
          src: "config/codeConversionCalculationFiles",
          dest: "codeConversionCalculationFiles",
        },
      ],
      serviceWorker: null,
    },
  ],
};
